/**
 * リスト2: BitMapCanvas.java
 * グラフィクス用の描画APIを提供するライブラリ
 */
public class BitMapCanvas{

    // 描画バックエンドのインスタンス．
    private final BitMapDisplay bitmap = new BitMapDisplay();

    public static final int BLACK     = 0x00000000;
    public static final int RED       = 0x000000FF;
    public static final int GREEN     = 0x0000FF00;
    public static final int BLUE      = 0x00FF0000;
    public static final int YELLOW    = 0x0000FFFF;
    public static final int PURPLE    = 0x00FF00FF;
    public static final int LIGHTBLUE = 0x00FFFF00;
    public static final int WHITE     = 0x00FFFFFF;

    // 四角形の描画
    public void rect(int x, int y, int w, int h, int color, int ctx){
	for(int i = 0; i < w; i++){
	    for(int j = 0; j < h; j++){
		pset(x+i, y+j, color, ctx);
	    }
	}
    }

    private int abs(int n){ if(n < 0){ return -n; }else{ return n; } }

    // Bresenham's アルゴリズムで直線を描画する
    public void line(int x0, int y0, int x1, int y1, int color, int ctx){
	int dx, dy, sx, sy;
	dx = abs(x1 - x0);
	sx = x0 < x1 ? 1 : -1;
	dy = -abs(y1 - y0);
	sy = y0 < y1 ? 1 : -1; 
	int err = dx + dy;
	int e2; /* error value e_xy */
	int x = x0;
	int y = y0;
	while(true){  /* loop */
	    pset(x, y, color, ctx);
	    if (x == x1 && y == y1) break;
	    e2 = 2*err;
	    if (e2 >= dy) { err += dy; x += sx; } /* e_xy+e_x > 0 */
	    if (e2 <= dx) { err += dx; y += sy; } /* e_xy+e_y < 0 */
	}
    }

    // 指定した座標の色を指定
    public void pset(int x, int y, int color, int ctx){
	int pos = x + y * 640;
	switch(ctx){
	case 0:
	    bitmap.ctx0[pos] = color;
	    break;
	case 1:
	    bitmap.ctx1[pos] = color;
	    break;
	default:
	    break;
	}
    }

    // フレームバッファの描画オフセットを指定する
    public void set_offset(int x, int y, int ctx){
	switch(ctx){
	case 0:
	    bitmap.ctx0_hoffset = x;
	    bitmap.ctx0_voffset = y;
	    break;
	case 1:
	    bitmap.ctx1_hoffset = x;
	    bitmap.ctx1_voffset = y;
	    break;
	default:
	    break;
	}
    }

}
