/**
 * リスト5: BitMapDispaly.java
 * ビットマップ画像用のメモリモジュール bitmap_display.vhd に
 * Javaからアクセスするためのブリッジクラス
 */
import java.util.EnumSet;
import synthesijer.hdl.HDLModule;
import synthesijer.hdl.HDLPort;
import synthesijer.hdl.HDLPrimitiveType;

// ラッパークラスはHDLModuleを継承して定義する
public class BitMapDisplay extends HDLModule{

    // Javaからアクセスするための変数群
    public int[] ctx0; // コンテクストメモリ0
    public int[] ctx1; // コンテクストメモリ1

    // フレームバッファ0の表示オフセット
    public int ctx0_voffset, ctx0_hoffset;
    // フーレムバッファ1の表示オフセット
    public int ctx1_voffset, ctx1_hoffset;

    // Synthesijerがクラスをロードするときに実行するコンストラクタ
    public BitMapDisplay(String... args){
	// ブリッジ対象がbitmap_displayであることを指定
	super("bitmap_display", "clk", "reset");
	// コンテクストメモリ用のポートを追加する
	genMemoryPorts("ctx0");
	genMemoryPorts("ctx1");

	// rgb出力用のポートを追加する
	// これらのポートはJavaの外の世界に引出すために，EXPORTオプションを指定する
	newPort("r", HDLPort.DIR.OUT, HDLPrimitiveType.genVectorType(8), EnumSet.of(HDLPort.OPTION.EXPORT));
	newPort("g", HDLPort.DIR.OUT, HDLPrimitiveType.genVectorType(8), EnumSet.of(HDLPort.OPTION.EXPORT));
	newPort("b", HDLPort.DIR.OUT, HDLPrimitiveType.genVectorType(8), EnumSet.of(HDLPort.OPTION.EXPORT));
	newPort("h_pos", HDLPort.DIR.IN, HDLPrimitiveType.genVectorType(32), EnumSet.of(HDLPort.OPTION.EXPORT));
	newPort("v_pos", HDLPort.DIR.IN, HDLPrimitiveType.genVectorType(32), EnumSet.of(HDLPort.OPTION.EXPORT));
    }

    // メモリ用のポート(アドレスやデータ入出力ポート)を生成するメソッド
    // これらはJavaの世界で使用するのでEXPORTオプションはつけない
    public void genMemoryPorts(String n){
	newPort(n+"_address", HDLPort.DIR.IN, HDLPrimitiveType.genSignedType(32));
	newPort(n+"_din",  HDLPort.DIR.IN, HDLPrimitiveType.genSignedType(32));
	newPort(n+"_dout", HDLPort.DIR.OUT, HDLPrimitiveType.genSignedType(32));
	newPort(n+"_we",   HDLPort.DIR.IN, HDLPrimitiveType.genBitType());
	newPort(n+"_oe",   HDLPort.DIR.IN, HDLPrimitiveType.genBitType());
	newPort(n+"_length", HDLPort.DIR.OUT, HDLPrimitiveType.genSignedType(32));
	newPort(n+"_voffset", HDLPort.DIR.IN, HDLPrimitiveType.genSignedType(32));
	newPort(n+"_hoffset", HDLPort.DIR.IN, HDLPrimitiveType.genSignedType(32));
    }
}
