/**
 * リスト3: BitMapDisplay.java
 * Swingを利用してソフトウェア上にBitMapDisplayを実装
 */
import java.awt.*;
import java.awt.image.BufferedImage;
import javax.swing.*;

// JFrameを継承することでウィンドウを生成する
public class BitMapDisplay extends JFrame{

    // フレームバッファ0と1
    public int[] ctx0 = new int[640*480];
    public int[] ctx1 = new int[640*480];

    // フレームバッファ0の表示オフセット
    public int ctx0_hoffset = 0, ctx0_voffset = 0;
    // フレームバッファ1の表示オフセット
    public int ctx1_hoffset = 0, ctx1_voffset = 0;

    // 描画領域
    TestCanvas canvas;

    // シンプルに表示ウィンドウを生成(あまり良いコードではない)．
    public BitMapDisplay(){
	canvas = new TestCanvas();
	getContentPane().add(canvas);
	pack();
	setVisible(true);
	(new FlashTimer()).start();
    }
	
    // 描画領域に相当するクラス
    class TestCanvas extends JPanel{
        // 描画用のイメージバッファ
	private BufferedImage img =
            new BufferedImage(640, 480, BufferedImage.TYPE_INT_RGB);
		
	TestCanvas(){
	    setSize(640, 480);
	    setPreferredSize(new Dimension(640, 480));
	}

	private int addOffset(int v, int o, int m){
	    int r = v + o;
	    return (r >= m) ? r - m : r;
	}
		
	public void paint(Graphics g){
	    super.paint(g);
            // フレームバッファの値を使って，VGA画像を生成する
	    for(int i = 0; i < 640; i++){
		for(int j = 0; j < 480; j++){
		    int i0 = addOffset(i, ctx0_hoffset, 640);
		    int j0 = addOffset(j, ctx0_voffset, 480);
		    int i1 = addOffset(i, ctx1_hoffset, 640);
		    int j1 = addOffset(j, ctx1_voffset, 480);

		    int r0 = (ctx0[(i0+j0*640)] & 0x000000FF) >> 0;
		    int g0 = (ctx0[(i0+j0*640)] & 0x0000FF00) >> 8;
		    int b0 = (ctx0[(i0+j0*640)] & 0x00FF0000) >> 16;
		    int r1 = (ctx1[(i1+j1*640)] & 0x000000FF) >> 0;
		    int g1 = (ctx1[(i1+j1*640)] & 0x0000FF00) >> 8;
		    int b1 = (ctx1[(i1+j1*640)] & 0x00FF0000) >> 16;
		    // フレームバッファ0を優先し，ピクセル情報を生成
		    int rb = ctx0[(i0+j0*640)] == 0 ? r1 : r0;
		    int gb = ctx0[(i0+j0*640)] == 0 ? g1 : g0;
		    int bb = ctx0[(i0+j0*640)] == 0 ? b1 : b0;
		    img.setRGB(i, j, (rb << 16) + (gb << 8) + bb);
		}
	    }
	    g.drawImage(img, 0, 0, this);
	}
    }
		
    // 適当なタイミングで自分自身リフレッシュするスレッド
    class FlashTimer extends Thread{
	public void run(){
	    while(true){
		canvas.repaint();
		try{
		    Thread.sleep(300);
		}catch(Exception e){
		    e.printStackTrace();
		}
	    }
	}
    }

}
